# Generated by Django 3.1.5 on 2021-01-08 19:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('strava', '0010_auto_20210108_1355'),
    ]

    operations = [
        migrations.AddField(
            model_name='activity',
            name='end_latitude',
            field=models.FloatField(null=True),
        ),
        migrations.AddField(
            model_name='activity',
            name='end_longitude',
            field=models.FloatField(null=True),
        ),
    ]



from django.urls import path

from . import views

urlpatterns = [
    path('login', views.login, name='stravalogin'),
    path('success', views.success, name='stravasuccess'),
    path('activity/<int:activity_id>/', views.activity, name='stravaactivitypage'),
    path('webhook', views.webhook),
]



# Generated by Django 3.1.5 on 2021-01-11 21:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('strava', '0011_auto_20210108_1405'),
    ]

    operations = [
        migrations.AddField(
            model_name='activity',
            name='fetched_detail',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='activity',
            name='fetched_streams',
            field=models.BooleanField(default=False),
        ),
    ]



# Create your models here.
from django.contrib.auth.models import User
from django.db import models


class StravaToken(models.Model):
    """Adds a token to a django User."""

    user = models.OneToOneField(User, on_delete=models.CASCADE)
    access_token = models.CharField(max_length=1024)
    refresh_token = models.CharField(max_length=1024)
    expires_in = models.IntegerField(null=True)
    expires_at = models.IntegerField(null=True)

    def __str__(self):
        return self.user.username


class StravaAthlete(models.Model):
    """Wraps around the API for an athletes summary:

    https://developers.strava.com/docs/reference/#api-models-SummaryAthlete"""

    user = models.OneToOneField(StravaToken, on_delete=models.CASCADE)
    last_updated = models.DateTimeField(null=True)
    all_data_fetched_initial = models.BooleanField(default=False)
    id = models.IntegerField(primary_key=True)
    username = models.CharField(max_length=1024, null=True)
    firstname = models.CharField(max_length=1024, null=True)
    lastname = models.CharField(max_length=1024, null=True)
    city = models.CharField(max_length=1024, null=True)
    state = models.CharField(max_length=1024, null=True)
    country = models.CharField(max_length=1024, null=True)
    sex = models.CharField(max_length=2, null=True)
    premium = models.BooleanField(default=False)
    # 'created_at': '2011-09-01T00:30:24Z',
    # 'updated_at': '2018-06-07T01:11:19Z',
    # 'badge_type_id': 1,
    profile_medium = models.URLField(max_length=1024, null=True)
    profile = models.URLField(max_length=1024, null=True)
    # 'friend': None,
    # 'follower': None,
    email = models.EmailField(max_length=1024, null=True)

    def __str__(self):
        return "%s %s (%s)" % (self.firstname, self.lastname, self.username)


class Activity(models.Model):
    """
    Based on:
    https://developers.strava.com/docs/reference/#api-models-SummaryActivity

    TODO: pull an activity summary, and an activity detail to see all possible fields.
    """

    athlete = models.ForeignKey(StravaAthlete, on_delete=models.CASCADE)

    # required
    id = models.IntegerField(primary_key=True)
    name = models.CharField(max_length=1024)
    type = models.CharField(max_length=200)
    start_date = models.DateTimeField()
    elapsed_time = models.IntegerField()
    fetched_detail = models.BooleanField(default=False)
    fetched_streams = models.BooleanField(default=False)

    # change to date
    start_date_local = models.DateTimeField(null=True)

    # the others, changed a few int -> float:
    elev_high = models.FloatField(null=True)
    average_temp = models.FloatField(null=True)
    total_photo_count = models.IntegerField(null=True)
    athlete_count = models.IntegerField(null=True)
    has_heartrate = models.BooleanField(default=False)
    total_elevation_gain = models.FloatField(null=True)
    average_cadence = models.FloatField(null=True)
    max_heartrate = models.FloatField(null=True)
    kudos_count = models.IntegerField(null=True)
    max_speed = models.FloatField(null=True)
    utc_offset = models.FloatField(null=True)
    gear_id = models.CharField(max_length=1024, null=True)
    location_country = models.CharField(max_length=1024, null=True)
    visibility = models.CharField(max_length=1024, null=True)
    comment_count = models.IntegerField(null=True)
    commute = models.BooleanField(default=False)
    private = models.BooleanField(default=False)
    moving_time = models.IntegerField(null=True)
    start_latitude = models.FloatField(null=True)
    start_longitude = models.FloatField(null=True)
    end_latitude = models.FloatField(null=True)
    end_longitude = models.FloatField(null=True)
    distance = models.FloatField(null=True)
    upload_id = models.IntegerField(null=True)
    average_speed = models.FloatField(null=True)
    elev_low = models.FloatField(null=True)
    timezone = models.CharField(max_length=1024, null=True)
    calories = models.FloatField(null=True)
    device_name = models.CharField(max_length=1024, null=True)
    description = models.TextField(null=True)
    trainer = models.BooleanField(default=False)
    achievement_count = models.IntegerField(null=True)
    pr_count = models.IntegerField(null=True)
    perceived_exertion = models.FloatField(null=True)
    flagged = models.BooleanField(default=False)
    average_heartrate = models.FloatField(null=True)
    manual = models.BooleanField(default=False)
    prefer_perceived_exertion = models.BooleanField(default=False, null=True)
    photo_count = models.IntegerField(null=True)
    has_kudoed = models.BooleanField(default=False)
    workout_type = models.IntegerField(null=True)
    average_watts = models.FloatField(null=True)
    kilojoules = models.FloatField(null=True)
    device_watts = models.BooleanField(default=False)
    max_watts = models.FloatField(null=True)
    weighted_average_watts = models.FloatField(null=True)

    # things I won't both to collect:
    # heartrate_opt_out = models.BooleanField(default=False)
    # display_hide_heartrate_option = models.BooleanField(default=False)
    # resource_state = models.IntegerField(null=True)
    # external_id = models.CharField(max_length=1024, null=True)
    # upload_id_str = models.CharField(max_length=1024, null=True)
    # from_accepted_tag = models.BooleanField(default=False)
    # embed_token = models.CharField(max_length=1024, null=True)

    # things with datatypes that I won't store:
    # {k for k, v in example_data.items() if type(v) == list}
    # {'available_zones',
    #  'best_efforts',
    #  'end_latlng',
    #  'laps',
    #  'segment_efforts',
    #  'splits_metric',
    #  'splits_standard',
    #  'start_latlng'}
    # {k for k, v in example_data.items() if type(v) == dict}
    # {'athlete', 'gear', 'map', 'photos', 'similar_activities'}

    def __str__(self):
        return self.name

    class Meta:
        ordering = ["-start_date"]

    """
    Based on:
    https://developers.strava.com/docs/reference/#api-models-StreamSet

    Should be returned by:
    https://developers.strava.com/docs/reference/#api-Streams-getActivityStreams
    """

    # could store the streams as binary numpy objects
    # see: https://stackoverflow.com/questions/46699238/how-to-make-a-numpy-array-field-in-django
    # binaryfield ref: https://docs.djangoproject.com/en/3.1/ref/models/fields/#binaryfield
    # could alternatively store them as JSON
    # https://docs.djangoproject.com/en/3.1/ref/models/fields/#jsonfield
    # will be simplest to stuff in the JSON
    # the numpy arrays could be faster in/out and doing computation

    # need at least time and distance
    time = models.JSONField(null=True)
    distance = models.JSONField(null=True)
    latlng = models.JSONField(null=True)
    altitude = models.JSONField(null=True)
    heartrate = models.JSONField(null=True)
    cadence = models.JSONField(null=True)
    watts = models.JSONField(null=True)
    temp = models.JSONField(null=True)

    # what is this one (maybe a boolean?):
    # - moving
    # don't need the smooths, compute them if necessary:
    # - velocity_smooth
    # - grade_smooth


    def get_example_data(user_index=0, n_activities=30):
        all_keys = set()
        example_data = dict()
        athlete = User.objects.all()[user_index].stravatoken.stravaathlete
        data = {"access_token": athlete.user.access_token}
        for a in athlete.activity_set.all()[:n_activities]:
            activity_id = a.id
            url = (
            f"https://www.strava.com/api/v3/activities/{activity_id}"
            + "?"
            + urllib.parse.urlencode({"include_all_efforts": False})
            )
            r = requests.get(url, data=data)
            activity = r.json()
            # print(activity)
            # print(keys - set(activity.keys()))
            # print(set(activity.keys()) - keys)
            all_keys = all_keys | set(activity.keys())
            for key in all_keys:
                if key in (activity.keys()) and key not in example_data:
                    example_data[key] = activity[key]

            {k: (v, type(v)) for k, v in example_data.items() if type(v) not in {list, dict, type(None)}}

            print('\n'.join([f'{k} = models.'+{int: 'IntegerField(null=True)', float: 'FloatField(null=True)', bool: 'BooleanField(default=False)', str: 'CharField(max_length=1024, null=True)'}[type(v)] for k, v in example_data.items() if type(v) not in {list, dict, type(None)}]))


class WebhookEvent(models.Model):
    class ObjectType(models.IntegerChoices):
        activity = 1
        athlete = 2

    class AspectType(models.IntegerChoices):
        create = 1
        update = 2
        delete = 3

    object_type = models.IntegerField(
        choices=ObjectType.choices, help_text='Always either "activity" or "athlete."'
    )
    aspect_type = models.IntegerField(
        choices=AspectType.choices, help_text='Always "create," "update," or "delete."'
    )
    object_id = models.BigIntegerField(
        help_text="For activity events, the activity's ID. For athlete events, the athlete's ID."
    )
    updates = models.JSONField(null=True)
    # updates = models.CharField(max_length=1000)
    owner_id = models.BigIntegerField(help_text="The athlete's ID.")
    subscription_id = models.IntegerField(
        help_text="The push subscription ID that is receiving this event."
    )
    event_time = models.BigIntegerField(help_text="The time that the event occurred.")
    processed = models.BooleanField(default=False)



# Generated by Django 3.1.5 on 2021-01-12 05:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('strava', '0013_auto_20210111_2104'),
    ]

    operations = [
        migrations.AddField(
            model_name='webhookevent',
            name='processed',
            field=models.BooleanField(default=False),
        ),
    ]



# Generated by Django 3.1.5 on 2021-01-07 20:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('strava', '0003_auto_20210106_2257'),
    ]

    operations = [
        migrations.CreateModel(
            name='WebhookEvents',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_type', models.IntegerField(choices=[(1, 'Activity'), (2, 'Athlete')], help_text='Always either "activity" or "athlete."')),
                ('aspect_type', models.IntegerField(choices=[(1, 'Create'), (2, 'Update'), (3, 'Delete')], help_text='Always "create," "update," or "delete."')),
                ('object_id', models.BigIntegerField(help_text="For activity events, the activity's ID. For athlete events, the athlete's ID.")),
                ('updates', models.JSONField()),
                ('owner_id', models.BigIntegerField(help_text="The athlete's ID.")),
                ('subscription_id', models.IntegerField(help_text='The push subscription ID that is receiving this event.')),
                ('event_time', models.BigIntegerField(help_text='The time that the event occurred.')),
            ],
        ),
    ]



# Generated by Django 3.1.5 on 2021-01-12 02:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('strava', '0012_auto_20210111_1622'),
    ]

    operations = [
        migrations.AlterField(
            model_name='webhookevent',
            name='updates',
            field=models.JSONField(null=True),
        ),
    ]



# Generated by Django 3.1.5 on 2021-01-08 04:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('strava', '0007_auto_20210107_2356'),
    ]

    operations = [
        migrations.AlterField(
            model_name='activity',
            name='elapsed_time',
            field=models.IntegerField(),
        ),
        migrations.AlterField(
            model_name='activity',
            name='name',
            field=models.CharField(max_length=1024),
        ),
        migrations.AlterField(
            model_name='activity',
            name='start_date',
            field=models.DateTimeField(),
        ),
        migrations.AlterField(
            model_name='activity',
            name='type',
            field=models.CharField(max_length=200),
        ),
    ]



# Generated by Django 3.1.5 on 2021-01-12 14:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('strava', '0014_webhookevent_processed'),
    ]

    operations = [
        migrations.AlterField(
            model_name='activity',
            name='prefer_perceived_exertion',
            field=models.BooleanField(default=False, null=True),
        ),
    ]



# Generated by Django 3.1.5 on 2021-01-07 20:40

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('strava', '0004_webhookevents'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='WebhookEvents',
            new_name='WebhookEvent',
        ),
    ]



# Generated by Django 3.1.5 on 2021-01-07 03:46

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='StravaToken',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('token', models.CharField(max_length=1024)),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='StravaAthlete',
            fields=[
                ('last_updated', models.DateTimeField(null=True)),
                ('all_data_fetched_initial', models.BooleanField(default=False)),
                ('id', models.IntegerField(primary_key=True, serialize=False)),
                ('username', models.CharField(max_length=1024, null=True)),
                ('firstname', models.CharField(max_length=1024, null=True)),
                ('lastname', models.CharField(max_length=1024, null=True)),
                ('city', models.CharField(max_length=1024, null=True)),
                ('state', models.CharField(max_length=1024, null=True)),
                ('country', models.CharField(max_length=1024, null=True)),
                ('sex', models.CharField(max_length=2, null=True)),
                ('premium', models.BooleanField(default=False)),
                ('profile_medium', models.URLField(max_length=1024, null=True)),
                ('profile', models.URLField(max_length=1024, null=True)),
                ('email', models.EmailField(max_length=1024, null=True)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='strava.stravatoken')),
            ],
        ),
        migrations.CreateModel(
            name='StravaActivity',
            fields=[
                ('id', models.IntegerField(primary_key=True, serialize=False)),
                ('external_id', models.CharField(max_length=1024, null=True)),
                ('upload_id', models.IntegerField(null=True)),
                ('name', models.CharField(max_length=1024, null=True)),
                ('distance', models.FloatField(null=True)),
                ('moving_time', models.IntegerField(null=True)),
                ('elapsed_time', models.IntegerField(null=True)),
                ('total_elevation_gain', models.FloatField(null=True)),
                ('elev_high', models.FloatField(null=True)),
                ('elev_low', models.FloatField(null=True)),
                ('start_date', models.DateTimeField(null=True)),
                ('start_date_local', models.DateTimeField(null=True)),
                ('timezone', models.CharField(max_length=1014, null=True)),
                ('achievement_count', models.IntegerField(default=0)),
                ('kudos_count', models.IntegerField(default=0)),
                ('comment_count', models.IntegerField(default=0)),
                ('athlete_count', models.IntegerField(default=0)),
                ('photo_count', models.IntegerField(default=0)),
                ('total_photo_count', models.IntegerField(default=0)),
                ('trainer', models.BooleanField(default=False)),
                ('commute', models.BooleanField(default=False)),
                ('manual', models.BooleanField(default=False)),
                ('private', models.BooleanField(default=False)),
                ('flagged', models.BooleanField(default=False)),
                ('workout_type', models.IntegerField(null=True)),
                ('average_speed', models.FloatField(null=True)),
                ('max_speed', models.FloatField(null=True)),
                ('athlete', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='strava.stravaathlete')),
            ],
        ),
    ]



# Generated by Django 3.1.5 on 2021-01-08 05:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('strava', '0008_auto_20210107_2357'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='activitystream',
            name='activity',
        ),
        migrations.RemoveField(
            model_name='activitysummary',
            name='activity',
        ),
        migrations.AddField(
            model_name='activity',
            name='achievement_count',
            field=models.IntegerField(default=0),
        ),
        migrations.AddField(
            model_name='activity',
            name='altitude',
            field=models.JSONField(null=True),
        ),
        migrations.AddField(
            model_name='activity',
            name='athlete_count',
            field=models.IntegerField(default=0),
        ),
        migrations.AddField(
            model_name='activity',
            name='average_speed',
            field=models.FloatField(null=True),
        ),
        migrations.AddField(
            model_name='activity',
            name='cadence',
            field=models.JSONField(null=True),
        ),
        migrations.AddField(
            model_name='activity',
            name='comment_count',
            field=models.IntegerField(default=0),
        ),
        migrations.AddField(
            model_name='activity',
            name='commute',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='activity',
            name='device_name',
            field=models.CharField(max_length=300, null=True),
        ),
        migrations.AddField(
            model_name='activity',
            name='elev_high',
            field=models.FloatField(null=True),
        ),
        migrations.AddField(
            model_name='activity',
            name='elev_low',
            field=models.FloatField(null=True),
        ),
        migrations.AddField(
            model_name='activity',
            name='flagged',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='activity',
            name='heartrate',
            field=models.JSONField(null=True),
        ),
        migrations.AddField(
            model_name='activity',
            name='kudos_count',
            field=models.IntegerField(default=0),
        ),
        migrations.AddField(
            model_name='activity',
            name='latlng',
            field=models.JSONField(null=True),
        ),
        migrations.AddField(
            model_name='activity',
            name='manual',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='activity',
            name='max_speed',
            field=models.FloatField(null=True),
        ),
        migrations.AddField(
            model_name='activity',
            name='photo_count',
            field=models.IntegerField(default=0),
        ),
        migrations.AddField(
            model_name='activity',
            name='private',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='activity',
            name='temp',
            field=models.JSONField(null=True),
        ),
        migrations.AddField(
            model_name='activity',
            name='time',
            field=models.JSONField(null=True),
        ),
        migrations.AddField(
            model_name='activity',
            name='timezone',
            field=models.CharField(max_length=1014, null=True),
        ),
        migrations.AddField(
            model_name='activity',
            name='total_photo_count',
            field=models.IntegerField(default=0),
        ),
        migrations.AddField(
            model_name='activity',
            name='trainer',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='activity',
            name='watts',
            field=models.JSONField(null=True),
        ),
        migrations.AddField(
            model_name='activity',
            name='workout_type',
            field=models.IntegerField(null=True),
        ),
        migrations.AlterField(
            model_name='activity',
            name='distance',
            field=models.JSONField(null=True),
        ),
        migrations.DeleteModel(
            name='ActivityDetail',
        ),
        migrations.DeleteModel(
            name='ActivityStream',
        ),
        migrations.DeleteModel(
            name='ActivitySummary',
        ),
    ]



# Generated by Django 3.1.5 on 2021-01-07 03:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('strava', '0002_auto_20210106_2253'),
    ]

    operations = [
        migrations.AlterField(
            model_name='stravatoken',
            name='expires_at',
            field=models.IntegerField(null=True),
        ),
        migrations.AlterField(
            model_name='stravatoken',
            name='expires_in',
            field=models.IntegerField(null=True),
        ),
    ]



# Generated by Django 3.1.5 on 2021-01-08 04:56

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('strava', '0006_auto_20210107_1554'),
    ]

    operations = [
        migrations.CreateModel(
            name='Activity',
            fields=[
                ('id', models.IntegerField(primary_key=True, serialize=False)),
                ('external_id', models.CharField(max_length=1024, null=True)),
                ('upload_id', models.IntegerField(null=True)),
                ('name', models.CharField(max_length=1024, null=True)),
                ('type', models.CharField(max_length=200, null=True)),
                ('start_date', models.DateTimeField(null=True)),
                ('start_date_local', models.DateTimeField(null=True)),
                ('distance', models.FloatField(null=True)),
                ('moving_time', models.IntegerField(null=True)),
                ('elapsed_time', models.IntegerField(null=True)),
                ('total_elevation_gain', models.FloatField(null=True)),
                ('athlete', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='strava.stravaathlete')),
            ],
            options={
                'ordering': ['-start_date'],
            },
        ),
        migrations.CreateModel(
            name='ActivityDetail',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('device_name', models.CharField(max_length=300, null=True)),
                ('activity', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='strava.activity')),
            ],
        ),
        migrations.CreateModel(
            name='ActivityStream',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('time', models.JSONField()),
                ('distance', models.JSONField()),
                ('latlng', models.JSONField(blank=True)),
                ('altitude', models.JSONField(blank=True)),
                ('heartrate', models.JSONField(blank=True)),
                ('cadence', models.JSONField(blank=True)),
                ('watts', models.JSONField(blank=True)),
                ('temp', models.JSONField(blank=True)),
                ('activity', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='strava.activity')),
            ],
        ),
        migrations.CreateModel(
            name='ActivitySummary',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('elev_high', models.FloatField(null=True)),
                ('elev_low', models.FloatField(null=True)),
                ('timezone', models.CharField(max_length=1014, null=True)),
                ('achievement_count', models.IntegerField(default=0)),
                ('kudos_count', models.IntegerField(default=0)),
                ('comment_count', models.IntegerField(default=0)),
                ('athlete_count', models.IntegerField(default=0)),
                ('photo_count', models.IntegerField(default=0)),
                ('total_photo_count', models.IntegerField(default=0)),
                ('trainer', models.BooleanField(default=False)),
                ('commute', models.BooleanField(default=False)),
                ('manual', models.BooleanField(default=False)),
                ('private', models.BooleanField(default=False)),
                ('flagged', models.BooleanField(default=False)),
                ('workout_type', models.IntegerField(null=True)),
                ('average_speed', models.FloatField(null=True)),
                ('max_speed', models.FloatField(null=True)),
                ('activity', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='strava.activity')),
            ],
        ),
        migrations.DeleteModel(
            name='StravaActivity',
        ),
    ]



# Generated by Django 3.1.5 on 2021-01-07 20:54

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('strava', '0005_auto_20210107_1540'),
    ]

    operations = [
        migrations.AlterField(
            model_name='stravaathlete',
            name='user',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='strava.stravatoken'),
        ),
    ]



# Generated by Django 3.1.5 on 2021-01-08 18:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('strava', '0009_auto_20210108_0004'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='activity',
            name='external_id',
        ),
        migrations.AddField(
            model_name='activity',
            name='average_cadence',
            field=models.FloatField(null=True),
        ),
        migrations.AddField(
            model_name='activity',
            name='average_heartrate',
            field=models.FloatField(null=True),
        ),
        migrations.AddField(
            model_name='activity',
            name='average_temp',
            field=models.FloatField(null=True),
        ),
        migrations.AddField(
            model_name='activity',
            name='average_watts',
            field=models.FloatField(null=True),
        ),
        migrations.AddField(
            model_name='activity',
            name='calories',
            field=models.FloatField(null=True),
        ),
        migrations.AddField(
            model_name='activity',
            name='description',
            field=models.TextField(null=True),
        ),
        migrations.AddField(
            model_name='activity',
            name='device_watts',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='activity',
            name='gear_id',
            field=models.CharField(max_length=1024, null=True),
        ),
        migrations.AddField(
            model_name='activity',
            name='has_heartrate',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='activity',
            name='has_kudoed',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='activity',
            name='kilojoules',
            field=models.FloatField(null=True),
        ),
        migrations.AddField(
            model_name='activity',
            name='location_country',
            field=models.CharField(max_length=1024, null=True),
        ),
        migrations.AddField(
            model_name='activity',
            name='max_heartrate',
            field=models.FloatField(null=True),
        ),
        migrations.AddField(
            model_name='activity',
            name='max_watts',
            field=models.FloatField(null=True),
        ),
        migrations.AddField(
            model_name='activity',
            name='perceived_exertion',
            field=models.FloatField(null=True),
        ),
        migrations.AddField(
            model_name='activity',
            name='pr_count',
            field=models.IntegerField(null=True),
        ),
        migrations.AddField(
            model_name='activity',
            name='prefer_perceived_exertion',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='activity',
            name='start_latitude',
            field=models.FloatField(null=True),
        ),
        migrations.AddField(
            model_name='activity',
            name='start_longitude',
            field=models.FloatField(null=True),
        ),
        migrations.AddField(
            model_name='activity',
            name='utc_offset',
            field=models.FloatField(null=True),
        ),
        migrations.AddField(
            model_name='activity',
            name='visibility',
            field=models.CharField(max_length=1024, null=True),
        ),
        migrations.AddField(
            model_name='activity',
            name='weighted_average_watts',
            field=models.FloatField(null=True),
        ),
        migrations.AlterField(
            model_name='activity',
            name='achievement_count',
            field=models.IntegerField(null=True),
        ),
        migrations.AlterField(
            model_name='activity',
            name='athlete_count',
            field=models.IntegerField(null=True),
        ),
        migrations.AlterField(
            model_name='activity',
            name='comment_count',
            field=models.IntegerField(null=True),
        ),
        migrations.AlterField(
            model_name='activity',
            name='device_name',
            field=models.CharField(max_length=1024, null=True),
        ),
        migrations.AlterField(
            model_name='activity',
            name='kudos_count',
            field=models.IntegerField(null=True),
        ),
        migrations.AlterField(
            model_name='activity',
            name='photo_count',
            field=models.IntegerField(null=True),
        ),
        migrations.AlterField(
            model_name='activity',
            name='timezone',
            field=models.CharField(max_length=1024, null=True),
        ),
        migrations.AlterField(
            model_name='activity',
            name='total_photo_count',
            field=models.IntegerField(null=True),
        ),
    ]



from django.contrib import admin

# Register your models here.
from .models import Activity, StravaAthlete, StravaToken, WebhookEvent

admin.site.register(StravaToken)
admin.site.register(StravaAthlete)
admin.site.register(Activity)
admin.site.register(WebhookEvent)



# Generated by Django 3.1.5 on 2021-01-07 03:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('strava', '0001_initial'),
    ]

    operations = [
        migrations.RenameField(
            model_name='stravatoken',
            old_name='token',
            new_name='access_token',
        ),
        migrations.AddField(
            model_name='stravatoken',
            name='expires_at',
            field=models.IntegerField(default=0),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='stravatoken',
            name='expires_in',
            field=models.IntegerField(default=0),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='stravatoken',
            name='refresh_token',
            field=models.CharField(default='', max_length=1024),
            preserve_default=False,
        ),
    ]



from django.test import TestCase


# Create your tests here.
def test_webhook():
    import json

    from .models import WebhookEvent

    response_text = """{
        "aspect_type": "update",
        "event_time": 1516126040,
        "object_id": 1360128428,
        "object_type": "activity",
        "owner_id": 134815,
        "subscription_id": 120475,
        "updates": {
            "title": "Messy"
        }
    }"""
    d = json.loads(response_text)
    d["object_type"] = {v.lower(): k for k, v in WebhookEvents.ObjectType.choices}[
        d["object_type"]
    ]
    d["aspect_type"] = {v.lower(): k for k, v in WebhookEvents.AspectType.choices}[
        d["aspect_type"]
    ]
    w = WebhookEvent(**d)
    w.save()



import datetime
import json
import logging
import urllib
import pytz
import threading
import copy

import requests
from django.contrib.auth import login
from django.contrib.auth.models import User
from django.http import HttpRequest, Http404, HttpResponse, HttpResponseRedirect, JsonResponse
from django.shortcuts import get_object_or_404, render
from django.views.decorators.csrf import csrf_exempt

from django.conf import settings
from .models import Activity, StravaAthlete, StravaToken, WebhookEvent

# reference on the strava handshake:
# https://developers.strava.com/docs/authentication/
# reference on subscribing to webhooks:
# https://developers.strava.com/docs/webhooks/
# reference on rate limits:
# https://developers.strava.com/docs/rate-limits/

# using threading for background tasks, could also do https://python-rq.org/

# Get an instance of a logger
logger = logging.getLogger(__name__)


def print_debug(_, debug=settings.DEBUG):
    logging.info(_)
    if debug:
        print(_)


def refresh_access_token(st: StravaToken) -> None:
    data = {
        "client_id": settings.STRAVA_CLIENT_ID,
        "client_secret": settings.STRAVA_CLIENT_SECRET,
        "refresh_token": st.refresh_token,
        "grant_type": "refresh_token",
    }

    # r = requests.post("https://www.strava.com/api/v3/oauth/token", data=data)
    r = requests.post("https://www.strava.com/oauth/token", data=data)
    # rather than raise an application error, pass that error back to the user:
    # r.raise_for_status()
    if r.status_code != requests.codes.ok:
        print_debug(f"Error in strava oauth handshake: {r.text}")

    response = r.json()
    print_debug(response)
    for key in {"access_token", "expires_at", "refresh_token"}:
        if key not in response:
            print_debug(f"No {key} in returned response: {response}.")
            return None

    st.access_token = response['access_token']
    st.expires_at = response['expires_at']
    st.refresh_token = response['refresh_token']
    # not confident we'll get this one back...
    if 'expires_in' in response:
        st.expires_in = response['expires_in']

    st.save()


def login(request: HttpRequest) -> HttpResponseRedirect:
    '''
    Redirect to authenticate to with Strava.
    '''
    data = {
        "client_id": settings.STRAVA_CLIENT_ID,
        "redirect_uri": "https://dashboard.aws.andyreagan.com/strava/success",
        "response_type": "code",
        "approval_prompt": "auto",
        "scope": "activity:read_all",
        "state": "",
    }
    if settings.DEBUG:
        data["redirect_uri"] = "http://localhost:8000/strava/success"
    return HttpResponseRedirect(
        "https://www.strava.com/oauth/authorize" + "?" + urllib.parse.urlencode(data)
    )


def save_activity_from_dict(activity: dict, athlete: StravaAthlete) -> None:
    '''
    Save an activity to an athlete's profile.
    Activity dict must contain the required fields of Activity model:
    - id: int
    - name: char
    - type: char
    - start_date: datetime
    - elapsed_time: int
    '''
    model_fields = {field.name for field in Activity._meta.get_fields()}
    activity["athlete"] = athlete

    # fix up dates
    # parse them both as UTC to start
    activity["start_date"] = datetime.datetime.strptime(
        activity["start_date"], "%Y-%m-%dT%H:%M:%SZ"
    ).replace(tzinfo=pytz.UTC)
    activity["start_date_local"] = datetime.datetime.strptime(
        activity["start_date_local"], "%Y-%m-%dT%H:%M:%SZ"
    ).replace(tzinfo=pytz.UTC)
    # then compute the offset and set that instead
    # there may be a utc_offset in the activity response, but I won't rely on it
    activity["start_date_local"] = activity["start_date_local"].replace(tzinfo=datetime.timezone(activity["start_date_local"] - activity["start_date"]))

    # flatten lat/long
    if 'start_latlng' in activity and type(activity.get('start_latlng')) == list and len(activity.get('start_latlng')) == 2:
        activity['start_latitude'] = activity['start_latlng'][0]
        activity['start_longitude'] = activity['start_latlng'][1]
    if 'end_latlng' in activity and type(activity.get('end_latlng')) == list and len(activity.get('end_latlng')) == 2:
        activity['end_latitude'] = activity['end_latlng'][0]
        activity['end_longitude'] = activity['end_latlng'][1]

    activity_fields = set(activity.keys())

    s = Activity(
        **{field: activity[field] for field in (model_fields & activity_fields)}
    )
    s.save()

    activity_missing = (model_fields - activity_fields)
    activity_extra = (activity_fields - model_fields)
    logger.info(f"{activity_missing=}")
    logger.info(f"{activity_extra=}")


def save_activities_from_response(activity_list: list, athlete: StravaAthlete) -> None:
    '''
    '''
    for activity in activity_list:
        save_activity_from_dict(activity, athlete)


def get_athlete_activities(athlete: StravaAthlete, max_requests: int=10) -> None:
    per_page = 200
    refresh_access_token(athlete.user)
    data = {
        "access_token": athlete.user.access_token,
    }

    more_activities = True
    i = 0
    while more_activities and (i < max_requests):
        i += 1
        url = (
            "https://www.strava.com/api/v3/athlete/activities"
            + "?"
            + urllib.parse.urlencode({"per_page": per_page, "page": i})
        )
        r = requests.get(url, data=data)
        activity_list = r.json()
        save_activities_from_response(activity_list, athlete)
        more_activities = len(activity_list) == per_page


def get_single_activity(owner_id: int, activity_id: int) -> None:
    athlete = StravaAthlete.objects.get(id=owner_id)
    refresh_access_token(athlete.user)
    data = {
        "access_token": athlete.user.access_token,
    }
    url = (
        f"https://www.strava.com/api/v3/activities/{activity_id}"
        + "?"
        + urllib.parse.urlencode({"include_all_efforts": False})
    )
    r = requests.get(url, data=data)
    activity = r.json()
    print_debug(activity)
    save_activity_from_dict(activity, athlete)


def success(request: HttpRequest) -> HttpResponse:
    if "code" not in request.GET:
        raise Http404 (f"No access code returned: {request.GET}.")

    data = {
        "client_id": settings.STRAVA_CLIENT_ID,
        "client_secret": settings.STRAVA_CLIENT_SECRET,
        "code": request.GET["code"],
        "grant_type": "authorization_code",
    }
    r = requests.post("https://www.strava.com/oauth/token", data=data)
    # rather than raise an application error, pass that error back to the user:
    # r.raise_for_status()
    if r.status_code != requests.codes.ok:
        raise Http404 (f"Error in strava oauth handshake: {r.text}")

    athlete = r.json()
    if "access_token" not in athlete:
        raise Http404 (f"No access token returned response: {athlete}.")

    # update these
    request.user.first_name = athlete["athlete"]["firstname"]
    request.user.last_name = athlete["athlete"]["lastname"]
    # request.user.email = athlete["athlete"]["email"]
    request.user.save()

    # update the token too
    (s, created) = StravaToken.objects.get_or_create(user=request.user)
    s.access_token = athlete.get("access_token")
    s.refresh_token = athlete.get("refresh_token")
    s.expires_in = athlete.get("expires_in")
    s.expires_at = athlete.get("expires_at")
    s.save()

    # save user
    fields = [x.name for x in StravaAthlete._meta.get_fields()]
    d = {f: athlete["athlete"][f] for f in fields if f in athlete["athlete"]}
    d["user"] = s
    (ath, created) = StravaAthlete.objects.get_or_create(**d)

    ath.save()
    # get_athlete_activities(ath, max_requests=1)
    t = threading.Thread(target=get_athlete_activities, args=[ath, 100])
    t.setDaemon(True)
    t.start()

    return render(request, "strava/success.html", athlete)


@csrf_exempt
def webhook(request: HttpRequest) -> HttpResponse:
    if request.method == "POST":
        logger.info(request.body)
        d = json.loads(request.body, strict=False)
        d["object_type"] = {v.lower(): k for k, v in WebhookEvent.ObjectType.choices}[
            d["object_type"]
        ]
        d["aspect_type"] = {v.lower(): k for k, v in WebhookEvent.AspectType.choices}[
            d["aspect_type"]
        ]
        w = WebhookEvent(**d)
        w.save()
        # pass the event id rather than the object (why?)
        t = threading.Thread(target=fetch_or_update,args=[w.id])
        t.setDaemon(True)
        t.start()
        return HttpResponse("")
    elif request.method == "GET":
        logger.info(request.GET)
        request.GET["hub.mode"] == "subscribe"
        challenge = request.GET["hub.challenge"]
        request.GET["hub.verify_token"] == "herecomescooldad"

        return JsonResponse({"hub.challenge": challenge})
    else:
        logger.error(request)
        raise Http404 ("Only know how to respond to GET and POST requests.")


def fetch_or_update(w_id: int) -> None:
    w = WebhookEvent.objects.get(id=w_id)
    if w.object_type == 1:  # activity = 1
        if w.aspect_type == 1:  # create = 1
            print_debug("create, so go get the activity")
            # go get the activity
            get_single_activity(owner_id=w.owner_id, activity_id=w.object_id)
            # TODO: run any other background tasks (refresh stats, etc)
        elif w.aspect_type == 2:  # update = 2
            print_debug("update, so go get the activity")
            # go fetch the activity from the db
            # update if we have it
            activities = Activity.objects.filter(id=w.object_id)
            if len(activities) == 1:
                print_debug("found it")
                activity = activities[0]
                model_fields = {field.name for field in Activity._meta.get_fields()}
                updates =  copy.deepcopy(w.updates)
                if 'title' in updates:
                    updates['name'] = updates['title']
                for k, v in updates.items():
                    if k in model_fields:
                        print_debug(f"setting field {k} in activity id {w.object_id}")
                        setattr(activity, k, v)
                    else:
                        print_debug(f"can't set field {k} in activity id {w.object_id}")
                        pass
                activity.save()
            else: # else go get it
                print_debug("it was an update, but we didn't have it, so go get it")
                get_single_activity(owner_id=w.owner_id, activity_id=w.object_id)
        else:  # delete = 3
            activities = Activity.objects.filter(id=w.object_id)
            if len(activities) == 1:
                activity = activities[0]
                activity.delete()
        w.processed = True
        w.save()
    else:  # athlete = 1
        pass


def activity(request: HttpRequest, activity_id: int) -> HttpResponse:
    this_activity = get_object_or_404(request.user.stravatoken.stravaathlete.activity_set, id=activity_id)
    activity_dict = {x.name: x.value_from_object(this_activity) for x in Activity._meta.get_fields()}
    return render(request, "strava/activity.html", {"activity": this_activity, "activity_dict": activity_dict})
